<?php

require("util.php");

function bluengo_install() {
	global $wpdb;
	global $bluengo_db_version;

	$table_name = $wpdb->prefix . 'bluengo_cache';

	$charset_collate = $wpdb->get_charset_collate();

	$sql = "CREATE TABLE $table_name (
		id BIGINT NOT NULL AUTO_INCREMENT,
		ts TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		type VARCHAR(16) NOT NULL,
		data TEXT NOT NULL,
		PRIMARY KEY  (id)
	) $charset_collate;";

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );

	add_option( 'bluengo_db_version', $bluengo_db_version );
	$emails = bluengo_get_administrator_email();
	$shop_name = get_the_title( get_option( 'woocommerce_shop_page_id' ) );

	$data = array(
		"uuid" => bluengo_uuid4(),
		"original_site_url" => get_site_url(),
		"original_install_ts" => time(),
		"original_admin_emails" => $emails,
		"original_shop_name" => $shop_name,
		"original_admin_ajax" => admin_url('admin-ajax.php') . "?action=bluengo_action",
		"integration_type" => "woocommerce_v2"
	);

	$link_result = bluengo_url_post_data( BLUENGO_BASE_API_URL . '/v1/tenant/link', $data);

	$data = array_merge($data, $link_result);

	$wpdb->insert(
		$table_name,
		array(
			'type' => "install",
			'data' => json_encode($data)
		)
	);

}

function bluengo_get_administrator_email() {
	$emails = [];
	$blogusers = get_users('role=Administrator');
	foreach ($blogusers as $user) {
		$emails[] = $user->user_email;
	}

	return $emails;
}

function bluengo_update_db_check() {
	global $bluengo_db_version;
	if ( get_site_option( 'bluengo_db_version' ) != $bluengo_db_version ) {
		bluengo_install();
	}
}

add_action( 'plugins_loaded', 'bluengo_update_db_check' );
